@@ -817,6 +817,7 @@ impl<'a> Iterator for FormatParser<'a> {
 #[derive(Debug)]
 pub struct Formatter {
     fields: StackVec<Field, MAX_FIELDS>,
+    format_exact: bool,
 }
 
 impl Formatter {
@@ -829,7 +830,9 @@ impl Formatter {
 
         for field in parser {
             if let Field::Invalid = field {
-                return Err(Error::InvalidFormat(fmt.as_ref().to_string()));
+                return Err(Error::InvalidFormat(
+                    "date format not recognized".to_string(),
+                ));
             }
 
             if fields.is_full() {
@@ -841,7 +844,10 @@ impl Formatter {
             fields.push(field);
         }
 
-        Ok(Formatter { fields })
+        Ok(Formatter {
+            fields,
+            format_exact: false,
+        })
     }
 
     /// Formats datetime types
@@ -961,6 +967,30 @@ impl Formatter {
     /// Parses datetime types
     #[inline]
     pub fn parse<S: AsRef<str>, T: DateTimeFormat>(&self, input: S) -> Result<T> {
+        let result = match self.format_exact {
+            true => self.parse_internal::<S, T, true>(input),
+            false => self.parse_internal::<S, T, false>(input),
+        };
+        if T::IS_INTERVAL_YM || T::IS_INTERVAL_DT {
+            match result {
+                Ok(_) => result,
+                Err(e) => match e {
+                    Error::ParseError(_) => {
+                        Err(Error::ParseError("the interval is invalid".to_string()))
+                    }
+                    _ => Err(e),
+                },
+            }
+        } else {
+            result
+        }
+    }
+
+    #[inline]
+    fn parse_internal<S: AsRef<str>, T: DateTimeFormat, const FX: bool>(
+        &self,
+        input: S,
+    ) -> Result<T> {
         let mut s = input.as_ref().as_bytes();
 
         let mut dt = if !T::HAS_DATE {
@@ -1025,14 +1055,13 @@ impl Formatter {
         let mut dow: Option<WeekDay> = None;
 
         for field in self.fields.iter() {
+            if !FX {
+                s = eat_whitespaces(s);
+            }
             match field {
                 // todo ignore the absence of symbols; Format exact
                 Field::Invalid => unreachable!(),
-                Field::Blank(n) => {
-                    for _ in 0..*n {
-                        expect_char_with_tolerence!(b' ')
-                    }
-                }
+                Field::Blank(_) => {}
                 Field::Hyphen => expect_char!(b'-'),
                 Field::Colon => expect_char_with_tolerence!(b':'),
                 Field::Slash => expect_char!(b'/'),
@@ -1140,7 +1169,7 @@ impl Formatter {
                             ));
                         }
                         let (hour, negative) = expect_number_with_tolerance!(T::HOUR_MAX_LENGTH, 0);
-                        if negative {
+                        if negative || hour < 1 || hour > 12 {
                             return Err(Error::ParseError(
                                 "hour must be between 1 and 12".to_string(),
                             ));
@@ -1269,6 +1298,10 @@ impl Formatter {
             }
         }
 
+        if !FX {
+            s = eat_whitespaces(s);
+        }
+
         if !s.is_empty() {
             return Err(Error::ParseError(
                 "format picture ends before converting entire input string".to_string(),
@@ -1331,6 +1364,12 @@ fn eat_digits(s: &[u8], max_len: usize) -> (&[u8], &[u8]) {
     (&s[..i], &s[i..])
 }
 
+#[inline]
+fn eat_whitespaces(s: &[u8]) -> &[u8] {
+    let i = s.iter().take_while(|&i| i.is_ascii_whitespace()).count();
+    &s[i..]
+}
+
 #[inline]
 fn parse_year(input: &[u8], max_len: usize, current_year: i32) -> Result<(bool, i32, &[u8])> {
     // todo do not allow sign element 's' before y/yy/yyy in the format string
@@ -1366,7 +1405,7 @@ fn parse_ampm(s: &[u8]) -> Result<(AmPm, &[u8])> {
     } else if CaseInsensitive::starts_with(s, b"P.M.") {
         Ok((AmPm::Pm, &s[4..]))
     } else {
-        Err(Error::ParseError("AM/PM is missing".to_string()))
+        Err(Error::ParseError("AM/A.M. or PM/P.M. required".to_string()))
     }
 }
 
@@ -1409,7 +1448,7 @@ fn parse_month_name(s: &[u8]) -> Result<(Month, &[u8])> {
         }
     }
 
-    Err(Error::ParseError("month is missing".to_string()))
+    Err(Error::ParseError("not a valid month".to_string()))
 }
 
 #[inline]
@@ -1426,7 +1465,7 @@ fn parse_week_day_name(s: &[u8]) -> Result<(WeekDay, &[u8])> {
         }
     }
 
-    Err(Error::ParseError("week day is missing".to_string()))
+    Err(Error::ParseError("not a valid day of the week".to_string()))
 }
 
 pub struct LazyFormat<T: DateTimeFormat> {
